#' date realted functions
#'
#' @import lubridate
#' @importFrom langeR get_lang
#' @importFrom tibble tibble
#'
#' @export
#'

get_datestr <- function(date, fmt = "%Y-%m-%d", tz = get_tz()) {
    date <- trydatetime(date, modestr = "D", tz = tz)
    return(format(date, fmt))
}

#'
#' @export
#'
get_W <- function(date, n = 1, tz = get_tz()) {
    date <- trydatetime(date, modestr = "D", tz = tz)
    return(sprintf("%02d", lubridate::yday(date) %/%  (7 * n)))
}

#'
#' @export
#'
get_weekdaydf <- function(lang = langeR::get_lang(),
                          firstweekday = ifelse(lang == "cn", "Monday", "Sunday"),
                          abbr = T,
                          abbr.short = F) {
    if (grepl("Sun|sun|日", firstweekday)) {
        levels  <- c("Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday")
        if (lang == "en") {
            if (isTRUE(abbr)) {
                if (isTRUE(abbr.short)) {
                    weekday  <- c("Su", "Mo", "Tu", "We", "Th", "Fr", "Sa")
                } else {
                    weekday  <- c("Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat")
                }
            } else {
                weekday  <- c("Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday")
            }
        } else if (lang == "cn") {
            if (isTRUE(abbr)) {
                if (isTRUE(abbr.short)) {
                    weekday  <- c("日", "一", "二", "三", "四", "五", "六")
                } else {
                    weekday  <- c("周日", "周一", "周二", "周三", "周四", "周五", "周六")
                }
            } else {
                weekday  <- c("星期日", "星期一", "星期二", "星期三", "星期四", "星期五", "星期六")
            }
        }
    } else {
        levels  <- c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday")
        if (lang == "en") {
            if (isTRUE(abbr)) {
                if (isTRUE(abbr.short)) {
                    weekday  <- c("Mo", "Tu", "We", "Th", "Fr", "Sa", "Su")
                } else {
                    weekday  <- c("Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun")
                }
            } else {
                weekday  <- c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday")
            }
        } else if (lang == "cn") {
            if (isTRUE(abbr)) {
                if (isTRUE(abbr.short)) {
                    weekday  <- c("一", "二", "三", "四", "五", "六", "日")
                } else {
                weekday  <- c("周一", "周二", "周三", "周四", "周五", "周六", "周日")
                }
            } else {
                weekday  <- c("星期一", "星期二", "星期三", "星期四", "星期五", "星期六", "星期日")
            }
        }
    }
    weekdaydf <- tibble::tibble(ind = 1:7, levels = levels, weekday = weekday)
    return(weekdaydf)
}

#'
#' @export
#'
get_wday <- function(date,
                     tz = get_tz(),
                     lang = langeR::get_lang(),
                     firstweekday = ifelse(lang == "cn", "Monday", "Sunday"),
                     abbr = T,
                     abbr.short = F,
                     weekdaydf = get_weekdaydf(lang = lang,
                                               firstweekday = firstweekday,
                                               abbr = abbr,
                                               abbr.short = abbr.short)
                     ) {
    date <- trydatetime(date, modestr = "D", tz = tz)
    return(factor(weekdays(date), levels = weekdaydf$levels, labels = weekdaydf$weekday))
}

#'
#' @export
#'
get_wdate <- function(date,
                      baseyear = lubridate::year(date[1]),
                      n = 1,
                      tz = get_tz()) {
    date <- trydatetime(date, modestr = "D", tz = tz)
    lubridate::year(date) <- baseyear
    if (n == 1) {
      # date <- lubridate::floor_date(date, unit = "week")
      date <- lubridate::ymd(paste0(baseyear, "0101")) +
        7 * n * floor((get_jday(date) - 1) / (7 * n))
    } else {
      date <- lubridate::ymd(paste0(baseyear, "0101")) +
        7 * n * floor((get_jday(date) - 1) / (7 * n))
    }
    return(date)
}

#'
#' @export
#'
YW2date <- function(YW, lang = langeR::get_lang()) {
    if (lang == "en") {
        Y <- floor(YW / 100)
        W <- YW - 100 * Y
    } else {
        Y <- parse_integer(str_extract(YW, "^\\d{4}"))
        W <- parse_integer(gsub("^.*(\\d{2})周$", "\\1", YW))
    }
    return(lubridate::ymd(paste0(Y, "-01-01")) + W * 7)
}

#'
#' @export
#'
get_Y <- function(date, lang = langeR::get_lang()) {
    if (!lubridate::is.Date(date)) {
        date <- lubridate::parse_date_time(date,
                                           orders = "ymd",
                                           tz = get_tz())
    }
    if (lang == "en") {
        return(paste0(format(date, "%Y")))
     }
    if (lang == "cn") {
        return(paste0(format(date, "%Y年")))
    }
}

#'
#' @export
#'
get_y <- function(date, lang = langeR::get_lang()) {
    if (!lubridate::is.Date(date)) {
        date <- lubridate::parse_date_time(date,
                                           orders = "ymd",
                                           tz = get_tz())
    }
    if (lang == "en") {
        return(paste0(format(date, "%y")))
     }
    if (lang == "cn") {
        return(paste0(format(date, "%y年")))
    }
}

#'
#' @export
#'
get_season <- function(date,
                       semi = "N",     # N for north, S for south.
                       tz = get_tz(),
                       lang = langeR::get_lang(),
                       abbr.short = F,
                       divisions = c("3-15", "6-15", "9-15","12-15")) {
    DIV <- paste0("2020-", divisions) %>% # using year 2000, since it is a leap year
        trydatetime(modestr = "D", tz = tz) %>%
        sort()
    if (semi == "N") {
        if (lang == "cn") {
            if (isTRUE(abbr.short)) {
                seasonname <- c("春", "夏", "秋", "冬")
            } else {
                seasonname <- c("春季", "夏季", "秋季", "冬季")
            }
        } else {
            if (isTRUE(abbr.short)) {
                seasonname <- c("Sp", "Su", "Au", "Wi")
            } else {
                seasonname <- c("Spring", "Summer", "Autumn", "Winter")
            }
        }
    } else {
        if (lang == "cn") {
            if (isTRUE(abbr.short)) {
                seasonname <- c("春", "夏", "秋", "冬")
            } else {
                seasonname <- c("春季", "夏季", "秋季", "冬季")
            }
        } else {
            if (isTRUE(abbr.short)) {
                seasonname <- c("Sp", "Su", "Au", "Wi")
            } else {
                seasonname <- c("Spring", "Summer", "Autumn", "Winter")
            }
        }
    }
    date %>%
        trydatetime(modestr = "D", tz = tz) %>%
        `year<-`(2020) %>%
        purrr::map_chr(~ifelse (.x >= max(DIV) | .x < min(DIV), seasonname[4],
                                ifelse (.x >= DIV[1] & .x < DIV[2], seasonname[1],
                                        ifelse (.x >= DIV[2] & .x < DIV[3], seasonname[2],
                                                seasonname[3])))) %>%
        factor(levels = seasonname) %>%
        return()
}

#'
#' @export
#'
get_S <- function(date, abbr.short = F, ...) {
    return(get_season(date, abbr.short = abbr.short, ...))
}

#'
#' @export
#'
get_s <- function(date, abbr.short = T, ...) {
    return(get_season(date, abbr.short = abbr.short, ...))
}

#'
#' @export
#'
get_seasoncn <- function(...) {
    return(get_season(lang = "cn", ...))
}

#'
#' @export
#'
get_YS <- function(date, lang = langeR::get_lang(), ...) {
    date <- trydatetime(date, modestr = "D", tz = tz)
    datesorted <- unique(sort(date))
    if (lang == "en") {
        YS <- factor(paste(get_Y(date, lang = "en"),
                           get_S(date, lang = "en", ...)),
                     levels = unique(paste(get_Y(datesorted, lang = "en"),
                           get_S(datesorted, lang = "en", ...))))
        return(Ys)
    }
    if (lang == "cn") {
        YS <- factor(paste0(get_Y(date, lang = "cn"),
                           get_S(date, lang = "cn", ...)),
                     levels = unique(paste0(get_Y(datesorted, lang = "cn"),
                           get_S(datesorted, lang = "cn", ...))))
        return(YS)
    }
}


#'
#' @export
#'
get_yS <- function(date, lang = langeR::get_lang(), ...) {
    date <- trydatetime(date, modestr = "D", tz = tz)
    datesorted <- unique(sort(date))
    if (lang == "en") {
        yS <- factor(paste(get_y(date, lang = "en"),
                           get_S(date, lang = "en", ...)),
                     levels = unique(paste(get_y(datesorted, lang = "en"),
                           get_S(datesorted, lang = "en", ...))))
        return(yS)
    }
    if (lang == "cn") {
        yS <- factor(paste0(get_y(date, lang = "cn"),
                           get_S(date, lang = "cn", ...)),
                     levels = unique(paste0(get_y(datesorted, lang = "cn"),
                           get_S(datesorted, lang = "cn", ...))))
        return(yS)
    }
}

#'
#' @export
#'
get_Ys <- function(date, lang = langeR::get_lang(), ...) {
    date <- trydatetime(date, modestr = "D", tz = tz)
    datesorted <- unique(sort(date))
    if (lang == "en") {
        Ys <- factor(paste(get_Y(date, lang = "en"),
                           get_s(date, lang = "en", ...)),
                     levels = unique(paste(get_Y(datesorted, lang = "en"),
                           get_s(datesorted, lang = "en", ...))))
        return(Ys)
    }
    if (lang == "cn") {
        Ys <- factor(paste0(get_Y(date, lang = "cn"),
                           get_s(date, lang = "cn", ...)),
                     levels = unique(paste0(get_Y(datesorted, lang = "cn"),
                           get_s(datesorted, lang = "cn", ...))))
        return(Ys)
    }
}


#'
#' @export
#'
get_ys <- function(date, lang = langeR::get_lang(), ...) {
    date <- trydatetime(date, modestr = "D", tz = tz)
    datesorted <- unique(sort(date))
    if (lang == "en") {
        ys <- factor(paste(get_y(date, lang = "en"),
                           get_s(date, lang = "en", ...)),
                     levels = unique(paste(get_y(datesorted, lang = "en"),
                           get_s(datesorted, lang = "en", ...))))
        return(ys)
    }
    if (lang == "cn") {
        ys <- factor(paste0(get_y(date, lang = "cn"),
                           get_s(date, lang = "cn", ...)),
                     levels = unique(paste0(get_y(datesorted, lang = "cn"),
                           get_s(datesorted, lang = "cn", ...))))
        return(ys)
    }
}

#'
#' @export
#'
get_jday <- function(date, tz = get_tz()) {
    date <- trydatetime(date, modestr = "D", tz = tz)
    return(lubridate::yday(date))
}

#'
#' @export
#'
get_jdate <- function(date, baseyear = lubridate::year(date[1]), tz = get_tz()) {
    date <- trydatetime(date, modestr = "D", tz = tz)
    lubridate::year(date) <- baseyear
    return(date)
}

#'
#' @export
#'
get_md <- function(date, lang = langeR::get_lang()) {
    if (lang == "en") {
        return(get_datestr(date, fmt = "%m-%d"))
    }
    if (lang == "cn") {
        return(get_datestr(date, fmt = "%m月%d日"))
    }
}

#'
#' @export
#'
get_Ymd <- function(date, lang = langeR::get_lang()) {
    if (lang == "en") {
        return(get_datestr(date, fmt = "%Y-%m-%d"))
    }
    if (lang == "cn") {
        return(get_datestr(date, fmt = "%Y年%m月%d日"))
    }
}

#'
#' @export
#'
get_Ymdcn <- function(date) {
        return(get_Ymd(date, lang = "cn"))
}

#'
#' @export
#'
get_ymd <- function(date, lang = langeR::get_lang()) {
    if (lang == "en") {
        return(get_datestr(date, fmt = "%y-%m-%d"))
    }
    if (lang == "cn") {
        return(get_datestr(date, fmt = "%y年%m月%d日"))
    }
}

#'
#' @export
#'
get_ymdcn <- function(date) {
        return(get_ymd(date, lang = "cn"))
}

#'
#' @export
#'
get_Ym <- function(date, lang = langeR::get_lang()) {
    if (lang == "en") {
        return(get_datestr(date, fmt = "%Y%m"))
    }
    if (lang == "cn") {
        return(get_datestr(date, fmt = "%Y年%m月"))
    }
}

#'
#' @export
#'
get_Ymcn <- function(date) {
        return(get_Ym(date, lang = "cn"))
}

#'
#' @export
#'
get_ym <- function(date, lang = langeR::get_lang()) {
    if (lang == "en") {
        return(get_datestr(date, fmt = "%y%m"))
    }
    if (lang == "cn") {
        return(get_datestr(date, fmt = "%y年%m月"))
    }
}

#'
#' @export
#'
get_ymcn <- function(date) {
        return(get_ym(date, lang = "cn"))
}

#'
#' @export
#'
get_Ymdshort <- function(date) {
    return(get_datestr(date, fmt = "%Y%m%d"))
}

#'
#' @export
#'
get_YW <- function(date, lang = langeR::get_lang()) {
    if (!lubridate::is.Date(date)) {
        date <- lubridate::parse_date_time(date,
                                           orders = "ymd",
                                           tz = get_tz())
    }
    if (lang == "en") {
        return(paste0(format(date, "%Y"),
                      sprintf("%02d", lubridate::yday(date) %/% 7)))
     }
    if (lang == "cn") {
        return(paste0(format(date, "%Y年"),
                      sprintf("第%02d周", lubridate::yday(date) %/% 7)))
    }
}

#'
#' @export
#'
get_yW <- function(date, lang = langeR::get_lang()) {
    if (!lubridate::is.Date(date)) {
        date <- lubridate::parse_date_time(date,
                                           orders = "ymd",
                                           tz = get_tz())
    }
    if (lang == "en") {
        return(paste0(format(date, "%y"), get_W(date)))
    }
    if (lang == "cn") {
        return(paste0(format(date, "%y年第"),
                      get_W(date),
                      "周"
                      ))
    }
}

#'
#' @export
#'
get_YWcn <- function(date) {
        return(get_YW(date, lang = "cn"))
}

#'
#' @export
#'
get_yWcn <- function(date) {
        return(get_yW(date, lang = "cn"))
}

#'
#' @export
#'
get_B <- function(date, lang = langeR::get_lang()) {
    if (lang == "en") {
        return(factor(month(date),
                      levels = 1:12,
                      labels = month.abb))
    }
    if (lang == "cn") {
        return(factor(month(date),
                      levels = c(1:12),
                      labels = paste0(1:12, "月")))
    }
}

#'
#' @export
#'
get_b <- function(...) {
    return(get_B(...))
}

#'
#' @export
#'
get_M <- function(date, fmt = `if`(lang == "en", "%b", "%m月"), lang = langeR::get_lang(), fct = T) {
    if (!lubridate::is.Date(date)) {
        date <- lubridate::parse_date_time(date,
                                           orders = "ymd",
                                           tz = get_tz())
    }
    if (lang == "en") {
        if (isTRUE(fct)) {
            return(factor(paste0(format(date, "%b")), levels = month.abb))
        }
        return(paste0(format(date, "%b")))
     }
    if (lang == "cn") {
        if (isTRUE(fct)) {
            return(factor(paste0(format(date, "%m月"))))
        }
        return(paste0(format(date, "%m月")))
    }
}

#'
#' @export
#'
get_m <- function(...) {
    return(get_M(...))
}

#'
#' @export
#'
get_bd <- function(date) {
    return(get_datestr(date, fmt = "%b %d"))
}

#'
#' @export
#'
get_Bd <- function(date) {
    return(get_datestr(date, fmt = "%B %d"))
}

#'
#' @export
#'
get_BdY <- function(date) {
    return(get_datestr(date, fmt = "%B %d, %Y"))
}

#'
#' @export
#'
get_Bdy <- function(date) {
    return(get_datestr(date, fmt = "%B %d, %y"))
}

#'
#' @export
#'
get_bdY <- function(date) {
    return(get_datestr(date, fmt = "%b %d, %Y"))
}

#'
#' @export
#'
get_bdy <- function(date) {
    return(get_datestr(date, fmt = "%b %d, %y"))
}

#'
#' @export
#'
get_BY <- function(date) {
    return(get_datestr(date, fmt = "%B, %Y"))
}

#'
#' @export
#'
get_By <- function(date) {
    return(get_datestr(date, fmt = "%B, %y"))
}

#'
#' @export
#'
get_bY <- function(date) {
    return(get_datestr(date, fmt = "%b, %Y"))
}

#'
#' @export
#'
get_by <- function(date) {
    return(get_datestr(date, fmt = "%b, %y"))
}

#'
#' @export
#'
get_Ystr <- function(date, lang = langeR::get_lang()) {
    if (lang == "cn") {
        return(get_datestr(date, fmt = "%Y年"))
    }
    return(get_datestr(date, fmt = "%Y"))
}

#'
#' @export
#'
get_Ystrcn <- function(date) {
    return(get_Ystr(date = date, lang = "cn"))
}

#'
#' @export
#'
get_mstr <- function(date, lang = langeR::get_lang()) {
    if (lang == "cn") {
        return(get_datestr(date, fmt = "%m月"))
    }
    return(get_datestr(date, fmt = "%m"))
}

#'
#' @export
#'
get_mstrcn <- function(date) {
    return(get_mstr(date = date, lang = "cn"))
}

#'
#' @export
#'
get_bstr <- function(date, lang = langeR::get_lang()) {
    if (lang == "cn") {
        return(get_datestr(date, fmt = "%m月"))
    }
    return(get_datestr(date, fmt = "%b"))
}

#'
#' @export
#'
get_bstrcn <- function(date) {
    return(get_bstr(date = date, lang = "cn"))
}

#'
#' @export
#'
get_Wstr <- function(date, lang = langeR::get_lang()) {
    if (lang == "cn") {
        return(paste0(get_W(date), "周"))
    }
    return(paste0("Week ", get_W(date)))
}

#'
#' @export
#'
get_Wstrcn <- function(date) {
    return(get_Wstr(date = date, lang = "cn"))
}
